package handler

import (
	"context"
	"strings"

	"google.golang.org/grpc/metadata"

	"git.loafle.net/commons_go/logging"
	oas "git.loafle.net/overflow/overflow_api_server/golang"
	ogw "git.loafle.net/overflow/overflow_gateway_websocket"
	"git.loafle.net/overflow/overflow_gateway_websocket/protocol/jsonrpc"
	ogp "git.loafle.net/overflow/overflow_grpc_pool"
	"go.uber.org/zap"
)

func newWebJSONRpcHandler(ctx context.Context, gPool ogp.Pool) ogw.ProtocolHandler {
	h := &webJSONRpcHandlers{
		ctx:    ctx,
		logger: logging.WithContext(ctx),
		gPool:  gPool,
	}

	p := jsonrpc.New(ctx, h)

	return p
}

type webJSONRpcHandlers struct {
	jsonrpc.JSONRpcHandlers
	ctx    context.Context
	logger *zap.Logger
	gPool  ogp.Pool
}

func (h *webJSONRpcHandlers) OnRequest(soc ogw.Socket, method string, params []string) (interface{}, error) {
	h.logger.Info("OnRequest",
		zap.String("path", soc.Path()),
		zap.String("method", method),
		zap.Any("params", params),
	)

	c, err := h.gPool.Get()
	if err != nil {
		h.logger.Error("cannot retrive GRPC Client")
		return nil, err
	}
	defer h.gPool.Put(c)

	sm := strings.Split(method, ".")

	si := &oas.ServerInput{
		Target: sm[0],
		Method: sm[1],
		Params: params,
	}

	md := metadata.Pairs("email", "overflow@loafle.com")
	ctx := metadata.NewOutgoingContext(context.Background(), md)

	so, err := c.(oas.OverflowApiServerClient).Exec(ctx, si)
	if err != nil {
		return nil, err
	}

	return so.Result, nil
}

func (h *webJSONRpcHandlers) OnNotify(soc ogw.Socket, method string, params []string) error {

	return nil
}
